#include <stdio.h>  
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <libharkio3.h>


#define PI 3.14159265358979

enum Type{ circle , grid , ball , cylinder} ;





//global 
typedef struct
{
	float min      ;
	float max      ;
	float interval ;
}arg_t;	 



typedef struct
{
	char  coord[24]   ;
	char  type[24]    ;
	char  output[1024];
	int   id          ;
	arg_t azimuth     ;
	arg_t elevation   ;
	arg_t height      ;
	arg_t radius      ;
	arg_t x_axis      ;
	arg_t y_axis      ;
	arg_t z_axis      ;
	char  path[1024]  ;
	harkio_Positions *poses;
}global_t;


global_t  g;



//Prototype
void MakeCirclePosition( );
void MakeGridPosition( );
void MakeBallPosition();
void MakeCylinderPosition();
void getArguments( int argc , char** argv );
void checkArguments();
void initGlobalVariable();
void MakefilePath(  float coord[] , char *output  );
void help(void);
void help_circle(void);
void help_grid(void);
void help_cylinder(void);
void help_ball(void);
void checkCircle(void);
void checkGrid(void);
void checkBall(void);
void checkCylinder(void);










int main(int argc, char **argv)
{

    harkio_XML *xml;
    FILE *fp;

	if( argc < 2)
	{
		harkio_Log_printf(harkio_Log_Error , "argument error!!" );
        help();
		exit(EXIT_FAILURE);
	}


	if( argc == 2 ) 
	{
		help();
		exit(EXIT_SUCCESS);
	}		
	else if( argc > 2 && (strcmp( argv[2]  ,  "--help") == 0) ) 
	{
		if( strcmp( argv[1]      , "circle") == 0 ) 
		{
			help_circle();
			exit(EXIT_SUCCESS);
		
		}
		if( strcmp( argv[1]      , "grid") == 0 ) 
		{
			help_grid();
			exit(EXIT_SUCCESS);
		
		}
		if( strcmp( argv[1]      , "cylinder") == 0 ) 
		{
			help_cylinder();
			exit(EXIT_SUCCESS);
		
		}
		if( strcmp( argv[1]      , "ball") == 0 ) 
		{
			help_ball();
			exit(EXIT_SUCCESS);
		}

	}



		
	// init
    initGlobalVariable();


	//get argument
	getArguments(  argc , argv  );

    checkArguments();


    if((fp = fopen( g.output , "w")) == NULL)
    {
        harkio_Log_FileOpenFailed( g.output );
        exit(EXIT_FAILURE);
    }


    g.poses = harkio_Positions_new( g.type , 0, 0, NULL);

    if( g.poses == NULL)
    {
        harkio_Log_printf(harkio_Log_Error,"can not harkio_Positions_new\n") ;
        exit(EXIT_FAILURE);
    }

    // parse argv using getopt
    if(strcmp( g.coord      , "circle") == 0)
    {
		MakeCirclePosition();
    }
    else if(strcmp(g.coord , "grid") == 0)
    {
		MakeGridPosition();
    }
    else if(strcmp(g.coord , "ball") == 0)
    {
        MakeCirclePosition();
    }
    else if(strcmp(g.coord , "cylinder") == 0)
    {
		MakeCylinderPosition();
    }
    else 
    {
        harkio_Log_printf(harkio_Log_Error,"Unknown coordinate type\n") ;
        exit(EXIT_FAILURE);
    }


    // make new XML object and write
    xml = harkio_XML_newFromData(g.poses, NULL, NULL);

    harkio_XML_write(xml, fp);

    harkio_XML_delete(&xml);

    fclose(fp);

    harkio_Log_printf(harkio_Log_Debug, "XML file created");


    exit(EXIT_SUCCESS);


}//mainend








/*--------------------------------------------------------//
 circle
	方向角（最少）
	方向角（最大）
	方向角（間隔）
    高さ  （最少）
    高さ  （最大）
    高さ  （間隔）
	半径（最少）
	半径（最大）
	半径（間隔）
--------------------------------------------------------*/
void MakeCirclePosition( )
{

	float azimuth   = 0 ;
    float radius    = 0 ;
    float elevation = 0;
	float coord[3]  ;
    float interval_elevatio ;
    float interval_radius   ;
    float interval_azimuth  ;
    harkio_Position *pos ;
    int id  ;
    char  pos_path[1024];
	char  tmp[256];



    //init
    id = g.id;
    interval_elevatio = g.elevation.interval;
    interval_radius   = g.radius.interval;
    interval_azimuth  = g.azimuth.interval;



    //interval 0 の場合
    if(g.elevation.interval == 0)
    {
        interval_elevatio = 16.70;
    }

    if(g.radius.interval == 0 )
    {
        interval_radius = 1;
    }

    if( g.azimuth.interval == 0 )
    {
        interval_azimuth = 5;
    }



	//radius
    radius = g.radius.min;
    while( radius <= g.radius.max )
    {
        //高さ方向(elevation)
        elevation = g.elevation.min;
        while( elevation <= g.elevation.max )
        {
            //方向角(azimuth)
            azimuth = g.azimuth.min;
            if( g.azimuth.min == 0 && g.azimuth.max == 360 ) g.azimuth.max = g.azimuth.max - interval_azimuth;

            while( azimuth <= g.azimuth.max )
            {
                sprintf(tmp, "[%d] azimuth[%f] elevation[%f] radius[%f]\n",id,azimuth,elevation,radius);
                harkio_Log_printf(harkio_Log_Debug, tmp);

                coord[0] = azimuth   ; //degree
                coord[1] = elevation ; //高さ
                coord[2] = radius    ; //メートル


                if( strlen(g.path) == 0 )
                {
                    pos = harkio_Position_new( id , Cartesian , coord , "" );
                }
                else
                {
                    MakefilePath(  coord ,  pos_path );
                    pos = harkio_Position_new( id , Cartesian , coord , pos_path );
                }


                if( pos == NULL)
                {
                    harkio_Log_printf(harkio_Log_Error,"can not harkio_Position_new\n") ;
                    exit(EXIT_FAILURE);
                }


                //極座標を直交座標 に変換する
                harkio_Position_Convert( pos , coord , Cartesian );

                //positionにセット
                pos->coord[0] = coord[0];
                pos->coord[1] = coord[1];
                pos->coord[2] = coord[2];

                harkio_Positions_append( g.poses , pos );
                if( id != -1)
                {
                    id++;
                }

                azimuth = azimuth + interval_azimuth;
            }

            elevation = elevation + interval_elevatio;

        }//while

        radius = radius +  interval_radius;
    }

}//end







/*--------------------------------------------------------//
grid
	X方向（最少）
	X方向（最大）
	X方向（間隔）
	y方向（最少）
	y方向（最大）
	y方向（間隔）
--------------------------------------------------------*/
void MakeGridPosition( )
{

    float x,y,z;
	float coord[3]  ;
    float interval_x_axis;
    float interval_y_axis;
    float interval_z_axis;
	char  tmp[256];
    char  pos_path[1024];
    harkio_Position *pos ;
    int   id;



    //init
    id = g.id;
    interval_x_axis = g.x_axis.interval;
    interval_y_axis = g.y_axis.interval;
    interval_z_axis = g.z_axis.interval;



    //interval 0 の場合
    if(g.x_axis.interval == 0)
    {
        interval_x_axis = 1;
    }

    if(g.y_axis.interval == 0 )
    {
        interval_y_axis = 1;
    }

    if( g.z_axis.interval == 0 )
    {
        interval_z_axis = 1;
    }



   z= g.z_axis.min;
   while( z <= g.z_axis.max)
   {
        x = g.x_axis.min;
        while( x <= g.x_axis.max)
        {
            y = g.y_axis.min;
            while( y <= g.y_axis.max )
            {
                coord[0] = x;
                coord[1] = y;
                coord[2] = z;

                sprintf(tmp, "[%d] = (%f, %f, %f)", id, x, y, z);
                harkio_Log_printf(harkio_Log_Debug, tmp);


                if( strlen(g.path) == 0 )
                {
                    pos = harkio_Position_new( id , Cartesian , coord , "" );
                }
                else
                {
                    MakefilePath(  coord ,  pos_path );
                    pos = harkio_Position_new( id , Cartesian , coord , pos_path );
                }

                if( pos == NULL)
                {
                    harkio_Log_printf(harkio_Log_Error,"can not harkio_Position_new\n") ;
                    exit(EXIT_FAILURE);
                }

                harkio_Positions_append( g.poses , pos );

                y += interval_y_axis;

                if( id != -1)
                {
                    id++;
                }
            }//y
            x += interval_x_axis;
        }//x
        z += interval_z_axis;
    }//z
}












//--------------------------------------------------------//
// cylinder
//方向角（最少）
//方向角（最大）
//方向角（間隔）
//高さ  （最少）
//高さ  （最大）
//高さ  （間隔）
//半径  （最少）
//半径  （最大）
//半径  （間隔）
//--------------------------------------------------------//
void MakeCylinderPosition( )
{


	float azimuth   ;
	float height    ;
	float radius    ;
    float azimuth_radian   ;
    float coord[3]  ;
    float interval_height ;
    float interval_radius   ;
    float interval_azimuth  ;
    char  pos_path[1024];
    harkio_Position *pos;
	int   id;


    //init
	id = g.id;
    interval_height   = g.height.interval;
    interval_radius   = g.radius.interval;
    interval_azimuth  = g.azimuth.interval;


    //interval 0 の場合
    if(g.height.interval == 0)
    {
        interval_height = 0.01;
    }

    if(g.radius.interval == 0 )
    {
        interval_radius = 1;
    }

    if( g.azimuth.interval == 0 )
    {
        interval_azimuth = 5;
    }



	//radius
    radius = g.radius.min;
    while( radius <= g.radius.max )
    {
		//高さ方向(height)
        height = g.height.min;
        while( height <= g.height.max )
        {
			//横方向(azimuth)
            azimuth = g.azimuth.min;
            while( azimuth <= g.azimuth.max )
            {
               // printf("[%d] azimuth[%f] height[%f] radius[%f]\n",id,azimuth,height,radius);

                azimuth_radian = harkio_Position_convToRadian(azimuth);

                coord[0] = radius * cosf(azimuth_radian) ;
                coord[1] = radius * sinf(azimuth_radian) ;
                coord[2] = height ;

                if( strlen(g.path) == 0 )
                {
                    pos = harkio_Position_new( id , Cartesian , coord , "" );
                }
                else
                {
                    MakefilePath(  coord ,  pos_path );
                    pos = harkio_Position_new( id , Cartesian , coord , pos_path );
                }


                if( pos == NULL)
                {
                    harkio_Log_printf(harkio_Log_Error,"can not harkio_Position_new\n") ;
                    exit(EXIT_FAILURE);
                }

			    harkio_Positions_append( g.poses , pos );

				if( id != -1)
				{
					id++;
				}

                azimuth = azimuth + interval_azimuth;
            }

            height = height + interval_height;
		}

        radius = radius + interval_radius ;
	}

}//











//--------------------------------------------------------//
//Make path

//--------------------------------------------------------//
void MakefilePath(   float coord[] , char *output  )
{


    int  cnt;
    char retValue[3][12];//xyz
    char charValue[12][1024];
    char input[1024];
    int  i;
    int  j;

    char cut[] = "{ } :";  /* 空白+ハイフン+ピリオド */
    char *tok ;
    char tmp[124];
    int  value;

    float azimuth;
    float elevation;
    float height;
    float radius;
    float x_axis;
    float y_axis;
    float z_axis;



    memset( charValue  ,'\0' , sizeof(charValue));
    memset( input  ,'\0' , sizeof(input));
    strcpy( input , g.path);


    tok = strtok( input , cut );

    cnt = 0;
    while( tok != NULL )
    {
        strcpy( charValue[cnt],tok);
        tok = strtok( NULL, cut );
        cnt++;
    }


    if(cnt>10)
    {
        harkio_Log_printf(harkio_Log_Error,"strtok count error\n") ;
        exit(EXIT_FAILURE);
    }


    memset(retValue ,'\0' , 12);
    memset( output  ,'\0' , ( size_t)(sizeof(output)));


    azimuth   = coord[0];
    elevation = coord[1];
    height    = coord[1];
    radius    = coord[2];
    x_axis    = coord[0];
    y_axis    = coord[1];
    z_axis    = coord[2];



    i=0;
    j=0;

    while( i<cnt )
    {
        memset(tmp      ,'\0' ,sizeof(tmp));

        if( strcmp( charValue[i] ,"AZIMUTH"  ) == 0 )
        {
            sprintf( tmp   ,  "%s" ,  charValue[i+1] );
            sprintf( retValue[j] , tmp , azimuth );
            j++;
        }
        else if( strcmp( charValue[i] ,"HEIGHT"  ) == 0 )
        {
            value = (int)(roundf( height * 1000));

            sprintf( tmp   ,  "%s" ,  charValue[i+1] );
            sprintf( retValue[j] , tmp , value );
            j++;
        }
        else if( strcmp( charValue[i]  ,"ELEVATION"  ) == 0 )
        {
            sprintf( tmp   ,  "%s" ,  charValue[i+1] );
            sprintf( retValue[j] , tmp , elevation );
            j++;

        }
        else if( strcmp( charValue[i]  ,"RADIUS"  ) == 0 )
        {
//            value = (int)(roundf(radius  * 1000 ));
            sprintf( tmp   ,  "%s" ,  charValue[i+1] );
            sprintf( retValue[j] , tmp , radius );
            j++;
       }
       else if( strcmp( charValue[i]  , "X_AXIS"  ) == 0 )
        {
            value = (int)(roundf(x_axis * 1000));

            sprintf( tmp   ,  "%s" ,  charValue[i+1] );
            sprintf( retValue[j] , tmp , value );
            j++;
        }
        else if( strcmp( charValue[i]  , "Y_AXIS"  ) == 0 )
        {
            value = (int)(roundf( y_axis * 1000));

            sprintf( tmp   ,  "%s" ,  charValue[i+1] );
            sprintf( retValue[j] , tmp , value );
            j++;

        }
        else if( strcmp( charValue[i]  , "Z_AXIS"  ) == 0 )
        {
            value = (int)(roundf( z_axis * 1000));

            sprintf( tmp   ,  "%s" ,  charValue[i+1] );
            sprintf( retValue[j] , tmp , value );
            j++;
        }

        i++;

    }//while




    //make file name
      strcpy( output,charValue[0]); //path

      //AZIMUTH
      strcat( output,retValue[0]);

      //ELEVVATION(HEIGHT)
      strcat( output,charValue[3]);
      strcat( output,retValue[1]);

      //RADIUS
      strcat( output,charValue[6]);
      strcat( output,retValue[2]);

      //other char
      strcat( output,charValue[9]);

//      printf("output[%s]\n",output);


}

















void initGlobalVariable()
{

    g.id                 = 0;
    g.azimuth.min        = 0;
    g.elevation.min      = 0;
    g.height.min         = 0;
    g.radius.min         = 0;
    g.x_axis.min         = 0;
    g.y_axis.min         = 0;
    g.z_axis.min         = 0;

    g.azimuth.max        = 0;
    g.elevation.max      = 0;
    g.height.max         = 0;
    g.radius.max         = 0;
    g.x_axis.max         = 0;
    g.y_axis.max         = 0;
    g.z_axis.max         = 0;

    g.azimuth.interval   = 0  ;
    g.elevation.interval = 0 ;
    g.height.interval    = 0  ;
    g.radius.interval    = 0  ;
    g.x_axis.interval    = 0  ;
    g.y_axis.interval    = 0  ;
    g.z_axis.interval    = 0  ;

    memset( g.coord  , '\0' , sizeof(g.coord) );
    memset( g.type   , '\0' , sizeof(g.type));
    memset( g.output , '\0' , sizeof(g.output));

}






void getArguments( int argc , char** argv )
{

    char *endptr;
    int cunt;


    printf("%s \n",argv[0]);

    strcpy(g.coord , argv[1]);

    printf("[ %s : ] \n", g.coord );


    cunt = 2;

    while( cunt < argc  )
    {
        if( strcmp( argv[cunt]      , "--type") == 0 )
        {
            cunt++;
            strcpy(g.type , argv[cunt]);
            printf("--type                 [%s]\n", g.type);
        }
        else if( strcmp( argv[cunt]      , "--output") == 0 ) 
        {
            cunt++;
            strcpy(g.output , argv[cunt]);
            printf("--output               [%s]\n", g.output);
        }
        //id
        else if( strcmp( argv[cunt]      , "--id") == 0 )
        {
            cunt++;
            g.id = atoi(argv[cunt] );
            printf("--id                   [%d]\n", g.id);
        }
        //path
        else if( strcmp( argv[cunt]      , "--path") == 0 )
        {
            cunt++;
           strcpy(g.path ,argv[cunt] );
            printf("--path                   [%s]\n", g.path);
        }
        //方向角（最少）
        else if( strcmp( argv[cunt]      , "--min-azimuth") == 0 )
        {
            cunt++;
            g.azimuth.min = strtof(argv[cunt] , &endptr );
            printf("--min-azimuth          [%f]\n", g.azimuth.min);
        }
        //方向角（最大）
        else if( strcmp( argv[cunt]      , "--max-azimuth") == 0 )
        {
            cunt++;
            g.azimuth.max = strtof(argv[cunt] , &endptr );
            printf("--max-azimuth          [%f]\n", g.azimuth.max);
        }
        //方向角（間隔）
        else if( strcmp( argv[cunt]      , "--interval-azimuth") == 0 )
        {
            cunt++;
            g.azimuth.interval = strtof(argv[cunt] , &endptr );
            printf("--interval-azimuth     [%f]\n", g.azimuth.interval);
        }

        //仰角  （最少）
        else if( strcmp( argv[cunt]      , "--min-elevation") == 0 )
        {
            cunt++;
            g.elevation.min = strtof(argv[cunt] , &endptr );
            printf("--min-elevation        [%f]\n", g.elevation.min);
        }
        //仰角  （最大）
        else if( strcmp( argv[cunt]      , "--max-elevation") == 0 )
        {
            cunt++;
            g.elevation.max = strtof(argv[cunt] , &endptr );
            printf("--max-elevation        [%f]\n", g.elevation.max);
        }
        //仰角  （間隔）
        else if( strcmp( argv[cunt]      , "--interval-elevation") == 0 )
        {
            cunt++;
            g.elevation.interval = strtof(argv[cunt] , &endptr );
            printf("--interval-elevation   [%f]\n", g.elevation.interval);
        }
        //高さ  （最少）
        else if( strcmp( argv[cunt]      , "--min-height") == 0 )
        {
            cunt++;
            g.height.min = strtof(argv[cunt] , &endptr );
            printf("--min-height           [%f]\n", g.height.min);
        }
        //高さ  （最大）
        else if( strcmp( argv[cunt]      , "--max-height") == 0 )
        {
            cunt++;
            g.height.max = strtof(argv[cunt] , &endptr );
            printf("--max-height           [%f]\n", g.height.max);
        }
        //高さ  （間隔）
        else if( strcmp( argv[cunt]      , "--interval-height") == 0 )
        {
            cunt++;
            g.height.interval = strtof(argv[cunt] , &endptr );
            printf("--interval-height      [%f]\n", g.height.interval);
        }
        //X方向（最少）
        else if( strcmp( argv[cunt]      , "--min-x_axis") == 0 )
        {
            cunt++;
            g.x_axis.min = strtof(argv[cunt] , &endptr );
            printf("--min-x_axis           [%f]\n", g.x_axis.min);
        }
        //X方向（最大）
        else if( strcmp( argv[cunt]      , "--max-x_axis") == 0 )
        {
            cunt++;
            g.x_axis.max = strtof(argv[cunt] , &endptr );
            printf("--max-x_axis           [%f]\n", g.x_axis.max);
        }
        //X方向（間隔）
        else if( strcmp( argv[cunt]      , "--interval-x_axis") == 0 )
        {
            cunt++;
            g.x_axis.interval = strtof(argv[cunt] , &endptr );
            printf("--interval-x_axis      [%f]\n", g.x_axis.interval);
        }
        //y方向（最少）
        else if( strcmp( argv[cunt]      , "--min-y_axis") == 0 )
        {
            cunt++;
            g.y_axis.min = strtof(argv[cunt] , &endptr );
            printf("--min-y_axis           [%f]\n", g.y_axis.min);
        }
        //y方向（最大）
        else if( strcmp( argv[cunt]      , "--max-y_axis") == 0 )
        {
            cunt++;
            g.y_axis.max = strtof(argv[cunt] , &endptr );
            printf("--max-y_axis           [%f]\n", g.y_axis.max);
        }
        //y方向（間隔）
        else if( strcmp( argv[cunt]      , "--interval-y_axis") == 0 )
        {
            cunt++;
            g.y_axis.interval = strtof(argv[cunt] , &endptr );
            printf("--interval-y_axis      [%f]\n", g.y_axis.interval);
        }
        //z方向（最少）
        else if( strcmp( argv[cunt]      , "--min-z_axis") == 0 )
        {
            cunt++;
            g.z_axis.min = strtof(argv[cunt] , &endptr );
            printf("--min-z_axis           [%f]\n", g.z_axis.min);
        }
        //z方向（最大）
        else if( strcmp( argv[cunt]      , "--max-z_axis") == 0 )
        {
            cunt++;
            g.z_axis.max = strtof(argv[cunt] , &endptr );
            printf("--max-z_axis           [%f]\n", g.z_axis.max);
        }
        //z方向（間隔）
        else if( strcmp( argv[cunt]      , "--interval-z_axis") == 0 )
        {
            cunt++;
            g.z_axis.interval = strtof(argv[cunt] , &endptr );
            printf("--interval-z_axis      [%f]\n", g.z_axis.interval);
        }
        //半径（最少）
        else if( strcmp( argv[cunt]      , "--min-radius") == 0 )  
        {
            cunt++;
            g.radius.min = strtof(argv[cunt] , &endptr );
            printf("--min-radius           [%f]\n", g.radius.min); 
        }
        //半径（最大）
        else if( strcmp( argv[cunt]      , "--max-radius") == 0 ) 
        {
            cunt++;
            g.radius.max = strtof(argv[cunt] , &endptr );
            printf("--max-radius           [%f]\n", g.radius.max);
        }
        //半径（間隔）
        else if( strcmp( argv[cunt]      , "--interval-radius") == 0 ) 
        {
            cunt++;
            g.radius.interval = strtof(argv[cunt] , &endptr );
            printf("--interval-radius      [%f]\n", g.radius.interval);
        }
	else
	{
		char msg[4192];
		sprintf(msg, "Unknown argument: %s", argv[cunt]);
	        harkio_Log_printf(harkio_Log_Error, msg) ;
	        help();
	        exit(EXIT_FAILURE);
	}
        cunt++;
    }

}//	









void checkArguments()
{

    //circle/grid
    if( strlen(g.coord)  == 0  )
    {
        harkio_Log_printf(harkio_Log_Error,"argument error [no coord]\n") ;
		exit(EXIT_FAILURE);
    }

    //tsp/microphon
    if( strlen(g.type)    == 0 )
    {
        harkio_Log_printf(harkio_Log_Error,"argument error [no type]\n") ;
		exit(EXIT_FAILURE);
    }

    if( strlen(g.output)  == 0 )
    {
        harkio_Log_printf(harkio_Log_Error,"argument error [no output]\n") ;
		exit(EXIT_FAILURE);
    }


    if(strcmp( g.coord      , "circle") == 0)
    {
		checkCircle();

    }
    else if(strcmp(g.coord , "grid") == 0)
    {
        checkGrid();
    }
    else if(strcmp(g.coord , "ball") == 0)
    {
        checkBall();
    }
    else if(strcmp(g.coord , "cylinder") == 0)
    {
        checkCylinder();
    }
	else
	{
        harkio_Log_printf(harkio_Log_Error,"coordinate type error\n") ;
		exit(EXIT_FAILURE);
	}

}









/*----------------------------------------------------------------*/
//
//
/*----------------------------------------------------------------*/
void checkCircle(void)
{

	//距離
	if( g.radius.min <= 0 )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [radius : min > 0] \n") ;
		exit(EXIT_FAILURE);
	}

	if( g.radius.max <= 0 )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [radius : max > 0] \n") ;
		exit(EXIT_FAILURE);
	}

	if( g.radius.min > g.radius.max )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [radius : min < max] \n") ;
		exit(EXIT_FAILURE);
	}

	//仰角
	if( g.elevation.min > g.elevation.max )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [elevation : min < max] \n") ;
		exit(EXIT_FAILURE);
	}

	if( g.elevation.interval == 0 )
	{
	    if( g.elevation.min != g.elevation.max )
	    {
	        harkio_Log_printf(harkio_Log_Error,"must be [elevation : min == max] \n") ;
			exit(EXIT_FAILURE);
	    }
	}

	//方向角
	if( g.azimuth.min >= g.azimuth.max )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [azimuth : min < max] \n") ;
		exit(EXIT_FAILURE);
	}

	if( g.azimuth.interval == 0 )
	{
		harkio_Log_printf(harkio_Log_Error,"must be [azimuth : interval > 0] \n") ;
		exit(EXIT_FAILURE);
	}

}//








/*----------------------------------------------------------------*/
//
//
/*----------------------------------------------------------------*/
void checkGrid(void)
{
	if( g.x_axis.min  >= g.x_axis.max )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [x_axis : min < max] \n") ;
		exit(EXIT_FAILURE);

	}

	if( g.x_axis.interval == 0 )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [x_axis : interval > 0] \n") ;
		exit(EXIT_FAILURE);
	}


	if( g.y_axis.min >= g.y_axis.max )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [y_axis : min < max] \n") ;
		exit(EXIT_FAILURE);
	}

	if(g.y_axis.interval == 0)
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [y_axis : interval > 0] \n") ;
		exit(EXIT_FAILURE);
	}

	if( g.z_axis.min > g.z_axis.max )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [z_axis : min < max] \n") ;
		exit(EXIT_FAILURE);
	}
}



/*----------------------------------------------------------------*/
//
//
/*----------------------------------------------------------------*/
void checkBall(void)
{
	//距離
	if( g.radius.min <= 0 )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [radius : min > 0] \n") ;
		exit(EXIT_FAILURE);
	}

	if( g.radius.max <= 0 )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [radius : max > 0] \n") ;
		exit(EXIT_FAILURE);
	}

	if( g.radius.min > g.radius.max )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [radius : min < max] \n") ;
		exit(EXIT_FAILURE);
	}

	//仰角
	 if( g.elevation.min > g.elevation.max )
	 {
	     harkio_Log_printf(harkio_Log_Error,"must be [elevation : min < max] \n") ;
		exit(EXIT_FAILURE);
	 }

	//方向角
	if( g.azimuth.min >= g.azimuth.max )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [azimuth : min < max] \n") ;
		exit(EXIT_FAILURE);
	}

	if( g.azimuth.interval == 0 )
	{
	   harkio_Log_printf(harkio_Log_Error,"must be [azimuth : interval > 0] \n") ;
		exit(EXIT_FAILURE);
	}
}



/*----------------------------------------------------------------*/
//
//
/*----------------------------------------------------------------*/
void checkCylinder(void)
{
	//距離
	if( g.radius.min <= 0 )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [radius : min > 0] \n") ;
		exit(EXIT_FAILURE);
	}

	if( g.radius.max <= 0 )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [radius : max > 0] \n") ;
		exit(EXIT_FAILURE);
	}

	if( g.radius.min > g.radius.max )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [radius : min < max] \n") ;
		exit(EXIT_FAILURE);
	}

	//高さ
	if( g.height.min > g.height.max )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [height : min < max] \n") ;
		exit(EXIT_FAILURE);
	}


	//方向角
	if( g.azimuth.min >= g.azimuth.max )
	{
	    harkio_Log_printf(harkio_Log_Error,"must be [azimuth : min < max] \n") ;
		exit(EXIT_FAILURE);
	}

	if( g.azimuth.interval == 0 )
	{
	   harkio_Log_printf(harkio_Log_Error,"must be [azimuth : interval > 0] \n") ;
		exit(EXIT_FAILURE);
	}
}
















/*----------------------------------------------------------------*/
//
//
/*----------------------------------------------------------------*/
void help(void)
{
	printf("                                                                            \n");
	printf("                                                                            \n");
	printf("[NAME:]                                                                     \n");
	printf("  harktoolcli-genposxml                                                     \n");
	printf("                                                                            \n");
	printf("[Description:]                                                              \n");
	printf("   create a position tag of xml file                                        \n");
	printf("                                                                            \n");
	printf("[Usage :]                                                                   \n");
    printf("   harktoolcli-genposxml [coordinate type] [argument]                       \n");
	printf("                                                                            \n");
	printf("[Usage of each coordinate type :]                                           \n");
    printf("   harktoolcli-genposxml circle   --help                                    \n");
    printf("   harktoolcli-genposxml grid     --help                                    \n");
    printf("   harktoolcli-genposxml cylinder --help                                    \n");
    printf("   harktoolcli-genposxml ball     --help                                    \n");
	printf("                                                                            \n");
	printf("                                                                            \n");
}



/*----------------------------------------------------------------*/
//
//
/*----------------------------------------------------------------*/
void help_circle(void)
{
	printf("                                                                            \n");
	printf("[coordinate type is circle:]                                                \n");
	printf("                                                                            \n");
    printf("  harktoolcli-genposxml circle  [Argument]                                  \n");
	printf("                                                                            \n");
    printf("[Argument :]                                                                \n");
	printf("   --type              Coordinate type                                      \n");
    printf("                        -tsp / impulse / noise / microphone / soundsource \n");
    printf("  --output             Output filename                                      \n");
    printf("  --id                 Position id of start                                 \n");
    printf("                        - id=-1:  all set [-1]  / id>=0 : increment        \n");
    printf("  --path               wav file path                                        \n");
    printf("                        - ex. file_name_D{AZIMUTH:%%03.1f}_E{ELEVATION:%%03.1f}_R{RADIUS:%%03.1f} \n");
    printf("  --min-azimuth        Minimum  of azimuth                                  \n");
    printf("  --max-azimuth        Maximum  of azimuth                                  \n");
    printf("  --interval-azimuth   Interval of azimuth                                  \n");
    printf("  --min-elevation      Minimum  of elevation                                \n");
    printf("  --max-elevation      Maximum  of elevation                                \n");
    printf("  --interval-elevation Interval of elevation                                \n");
    printf("  --min-radius         Minimum  of radius                                   \n");
    printf("  --max-radius         Maximum  of radius                                   \n");
    printf("  --interval-radius    Interval of radius                                   \n");
	printf("                                                                            \n");
}





/*----------------------------------------------------------------*/
//
//
/*----------------------------------------------------------------*/
void help_grid(void)
{
	printf("                                                                            \n");
    printf("[coordinate type is grid:]                                                  \n");
	printf("                                                                            \n");
    printf("  harktoolcli-genposxml grid  [Argument]                                    \n");
	printf("                                                                            \n");
    printf("[Argument :]                                                                \n");
	printf("  --type               Coordinate type                                      \n");
    printf("                        - tsp / impulse / noise / microphone / soundsource \n");
    printf("  --output             Output filename                                      \n");
    printf("  --id                 Position id of start                                 \n");
    printf("                        - id=-1:  all set [-1]  / id>=0 : increment        \n");
    printf("  --path               position path                                        \n");
    printf("  --min-x_axis         Minimum  of x_axis                                   \n");
    printf("  --max-x_axis         Maximum  of x_axis                                   \n");
    printf("  --interval-x_axis    Interval of x_axis                                   \n");
    printf("  --min-y_axis         Minimum  of y_axis                                   \n");
    printf("  --max-y_axis         Maximum  of y_axis                                   \n");
    printf("  --interval-y_axis    Interval of y_axis                                   \n");
    printf("  --min-z_axis         Minimum  of y_axis                                   \n");
    printf("  --max-z_axis         Maximum  of y_axis                                   \n");
    printf("  --interval-z_axis    Interval of y_axis                                   \n");
	printf("                                                                            \n");
}






/*----------------------------------------------------------------*/
//
//
/*----------------------------------------------------------------*/
void help_cylinder(void)
{
	printf("                                                                            \n");
    printf("[coordinate type is cylinder:]                                              \n");
	printf("                                                                            \n");
    printf("  harktoolcli-genposxml cylinder  [Argument]                                \n");
	printf("                                                                            \n");
    printf("[Argument :]                                                                \n");
	printf("  --type               Coordinate type                                      \n");
    printf("                        - tsp / impulse / noise / microphone / soundsource \n");
    printf("  --output             Output filename                                      \n");
    printf("  --id                 Position id of start                                 \n");
    printf("                        - id=-1:  all set [-1]  / id>=0 : increment        \n");
    printf("  --path               position path                                        \n");
    printf("  --min-azimuth        Minimum  of azimuth                                  \n");
    printf("  --max-azimuth        Maximum  of azimuth                                  \n");
    printf("  --interval-azimuth   Interval of azimuth                                  \n");
    printf("  --min-height         Minimum  of height                                   \n");
    printf("  --max-height         Maximum  of height                                   \n");
    printf("  --interval-height    Interval of height                                   \n");
    printf("  --min-radius         Minimum  of radius                                   \n");
    printf("  --max-radius         Maximum  of radius                                   \n");
    printf("  --interval-radius    Interval of radius                                   \n");
	printf("                                                                            \n");
}




/*----------------------------------------------------------------*/
//
//
/*----------------------------------------------------------------*/
void help_ball(void)
{
	printf("                                                                            \n");
    printf("[coordinate type is ball:]                                                  \n");
	printf("                                                                            \n");
    printf("  harktoolcli-genposxml ball  [Argument]                                    \n");
	printf("                                                                            \n");
    printf("[Argument :]                                                                \n");
	printf("  --type               Coordinate type                                      \n");
    printf("                        - tsp / impulse / noise / microphone / soundsource \n");
    printf("  --output             Output filename                                      \n");
    printf("  --id                 Position id of start                                 \n");
    printf("                        - id=-1:  all set [-1]  / id>=0 : increment        \n");
    printf("  --path               position path                                        \n");
    printf("  --min-azimuth        Minimum  of azimuth                                  \n");
    printf("  --max-azimuth        Maximum  of azimuth                                  \n");
    printf("  --interval-azimuth   Interval of azimuth                                  \n");
    printf("  --min-elevation      Minimum  of elevation                                \n");
    printf("  --max-elevation      Maximum  of elevation                                \n");
    printf("  --interval-elevation Interval of elevation                                \n");
    printf("  --min-radius         Minimum  of radius                                   \n");
    printf("  --max-radius         Maximum  of radius                                   \n");
    printf("  --interval-radius    Interval of radius                                   \n");
	printf("                                                                            \n");
}





