/*-----------------------------------------------------------------------------*
*
*  calctf.c をinclude すること
*
*
*-----------------------------------------------------------------------------*/
#include <stdio.h>  
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <libharkio3.h>
#include <math.h>

#include "harktool.h"







#define ZIP_FILE_MAX  7


void StrSort(char *str[ ], int n);

float calcZValue( int z_unit , float re , float im );
void WriteCoordinate(  char *outfile , int z_axis , int fs);
void getZipData( struct zip  *handle , char *filename  ,int *rows ,  int* cols ,int *nfs);
void calcIFFT_TF(  struct zip  *handle ,char *fname[] ,int f_max ) ;
void global_init();






#define DISP_REAL     1
#define DISP_AMP     2
#define DISP_db     3
#define DISP_PHASE  4


#define  LOCDIR  "transferFunction/localization/tf"
#define	 SEPDIR  "transferFunction/separation/tf"

#define COMPLEX_LEN 8



//harktool global 構造体
struct GLOBAL  g;

float *float_a;



void help(void)
{
	printf("                                                \n");
	printf("                                                \n");
	printf("[NAME:]                                         \n");
	printf("  harktoolcli-ifftTF                         \n");
	printf("                                                \n");
	printf("[Description:]                                  \n");
	printf("  Coordinate from transfer function             \n");
	printf("                                                \n");
	printf("[Usage :]                                       \n");
    printf("  harktoolcli-ifftTF tf_type tf_filename(zip)  \n");
	printf("                                                  \n");
    printf("[options]                                                     \n");
    printf("      -t: Type of Transfer function    ex.(-t LOC)    [string]   \n");
 	printf("            LOC : Localization transfer function              \n");
 	printf("            SEP : Separation   transfer function              \n");
//    printf("      -z: z_axis type   ( amp: amplitude   db: decibel  phase:phase  [string]  \n");//ifft is amp only
    printf("      -i: input file                     [string]    \n");
    printf("      -o: output file                    [string]    \n");
//    printf("      -s: Sampling frequency           (-s 16000)    [integer] \n");
//    printf("      -n: Number of fft                (-n 512)      [integer] \n");
	printf("                                                               \n");

}









int main(int argc, char **argv)
{
    int option;

    int        errorp;
    int        index;
    const char *name;
    char msg[MSG_LEN];
    int  flags = 0;
    struct zip  *handle;
    zip_int64_t  num;

    memset(msg,  '\0' , sizeof(msg));


    char infile[2048];
    char outfile[2048];
	char tfdirname[1024];

	int samplingrate =0;
	int z_axis = DISP_REAL;
	int tf_type=0;

	//char *fname[20];
	char tmp[10][100];
	memset(tmp,0,sizeof(tmp));



    if(argc < 2)
    {
		harkio_Log_printf(harkio_Log_Error , "argument error!!" );
        help();
		exit(EXIT_FAILURE);
    }


    // parse option
    while((option = getopt(argc, argv, "t:z:o:i:s:n:")) != -1)
    {
        switch(option)
        {
        case 't':
        	if( strcmp( optarg , "LOC" ) == 0  ||  strcmp( optarg , "loc" ) == 0 )
        	{
				memset( tfdirname , 0x00 ,sizeof(tfdirname) );
	        	strcpy( tfdirname , "transferFunction/localization/tf" );
	        	tf_type=0;
        	}
        	else if( strcmp( optarg , "SEP" ) == 0 || strcmp( optarg , "sep" ) == 0 )
			{
				memset( tfdirname , 0x00 ,sizeof(tfdirname) );
	        	strcpy( tfdirname , "transferFunction/separation/tf" );
	        	tf_type=1;
			}            
			else
			{
		        help();
			    exit(EXIT_FAILURE);
			}

        break;

        case 'z':
        	if( strcmp( optarg , "amp" ) == 0 )
        	{
        		z_axis  = DISP_AMP;
        	}
        	else if( strcmp( optarg , "db" ) == 0 )
			{
        		z_axis  = DISP_db;
			}            
        	else if( strcmp( optarg , "phase" ) == 0 )
			{
        		z_axis  = DISP_PHASE;
			}            
			else
			{
		        help();
			    exit(EXIT_FAILURE);
			}

        break;

        case 'i':
			memset( infile , 0x00 , sizeof(infile));
	        strncpy( infile , optarg, strlen(optarg));
        break;

        case 'o':
			memset( outfile , 0x00 , sizeof(outfile));
	        strncpy( outfile , optarg, strlen(optarg));
	        break;

//        case 's':
//			samplingrate = atoi(optarg);
//	        break;

//        case 'n':
//			fft_len = atoi(optarg);
//	        break;


        case '?':
            help();
	        exit(EXIT_FAILURE);
        }
    }


	global_init();


    /*--------------------------------------------------*/
    /* 領域確保                                         */
    /*--------------------------------------------------*/
    // 伝達関数用領域
    // npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の領域確保
	//  g.Nsrc , g.Nmic , (g.Nfft/2+1)
 
    //SORT
    harktool_new( SORT_POS_ID );



    /*--------------------------------------------------*/
    //  FFT data set
    /*--------------------------------------------------*/
    setFftData();




    handle = zip_open( infile , ZIP_CHECKCONS, &errorp);


    if(handle == 0x00 )
    {
        sprintf(msg,"can not open zip file [%s]",argv[1]);
        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
    }

    num = zip_get_num_entries( handle ,  flags);


    if( num < ZIP_FILE_MAX )
    {
        sprintf(msg,"file count  is different [%s]",argv[1]);
        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
    }




	char *fname[num];
	int cnt=0;
	
    for(index=0 ; index<num ; index++)
    {
		name = zip_get_name( handle , index , flags);

		if(name == NULL) continue;		

		if( strncmp( tfdirname , name , strlen(tfdirname) ) == 0)
		{
	    	fname[cnt] = (char*)name;
			cnt++;
		}
    }


    if( cnt == 0 )
    {
    	switch(tf_type)
    	{
    	case 0:
	        sprintf(msg , "not mat file of localization\n" );
			break;			
		case 1:
	        sprintf(msg , "not mat file of separation\n" );
			break;			
		}
		
        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
    }




    /*--------------------------------------------------*/
    //音源数
    /*--------------------------------------------------*/
    g.Nsrc = cnt;




	/*---------------------------------------*/
	//sort
	/*---------------------------------------*/
	StrSort( fname , cnt);


	/* ソート結果を表示 */
	int i;
	printf("\nソート後: \n");		
	for (i = 0; i < cnt; i++)
	{
		printf("%s  \n", fname[i]);
	}
	printf("\n");



	
	/*---------------------------------------*/
	// nmic , nfft get
	/*---------------------------------------*/
	int rows = 0;
	int nfft = 0;
	int nfs   = 0;

	getZipData( handle , fname[0] , &rows , &nfft , &nfs);
	samplingrate = nfs;

//	if( samplingrate == 0 ) samplingrate = nfs;

	/*---------------------------------------*/
	//mic数
	/*---------------------------------------*/
	g.Nmic = rows;



	/*---------------------------------------*/
	// nfft
	/*---------------------------------------*/
	g.Nfft =  nfft; //512





	/*-----------------------------------------------------------------------------*
	* ３次元配列を作成する(complex)
	* IN : size_x(Nsrc) x size_y(Nmic) x size_z( Nfft/2+1 )
	//g.TFvect[src][mic][fft]
	*-----------------------------------------------------------------------------*/
	g.TFvect = harktool_newComplexThreeDim(  g.Nsrc , g.Nmic , g.Nfft );


	//伝達関数作業用３次元配列を作成する(complex)
	//*  Nsrc x Nmic x Nfft/2+1
	// 伝達関数 作業用領域
	// npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の作業領域確保
	harktool_new( TMP_TF_VECT );


	//伝達関数作成用TEMP領域
	harktool_new( BUFFER );
   

	//TEMP領域
	int len = g.Nfft * sizeof(float);
	float_a = (float*)malloc( len );

    if( float_a == NULL )
    {
        harktool_LogE("malloc error" );
        harktool_Exit(EXIT_FAILURE);
    }

	memset( (void*)float_a , 0 , len);




	/*---------------------------------------*/
	// data get
	/*---------------------------------------*/
	calcIFFT_TF(  handle ,  fname , g.Nsrc );
	


	/*---------------------------------------*/
	// file write
	/*---------------------------------------*/
	WriteCoordinate(  outfile , z_axis , samplingrate);



	/*---------------------------------------*/
	// exit
	/*---------------------------------------*/
	printf("SUCCESS\n");
	
	zip_close(handle);
	exit(EXIT_SUCCESS);

}








/*------------------------------------------------------*/
//
/* n個の文字列をソート */
//void StrSort(char *str[ ], int n)
/*------------------------------------------------------*/
void StrSort(char *str[], int n)
{
    int i, j;
    char *temp;   /* 作業用ポインタ */
	
    for (i = 0; i < n - 1; i++) {
        for (j = i + 1; j < n; j++) {
            if ((strcmp(str[j], str[i])) < 0) {
                temp = str[i];                /* 作業用ポインタを使い */
                str[i] = str[j];              /* i + 1番目に小さい要素と */
                str[j] = temp;                /* str[i] の要素と交換 */
            }
        }

    }
}








/*---------------------------------------*/
// Z軸計算
/*---------------------------------------*/
float calcZValue( int z_unit , float re , float im   )
{

    char msg[MSG_LEN];


    switch(z_unit)
    {
        case DISP_REAL: 
            return re;
            break;

        case DISP_AMP:
        return sqrtf(re*re + im*im);
            break;
        case DISP_db:
            return log10(sqrtf(re*re + im*im)) * 10;
            break;
        case DISP_PHASE:
            return atan2f(im, re) * 180/M_PI;
            break;
        default:
		    sprintf(msg,"z_unit error [%d]", z_unit );
		    harkio_Log_printf(harkio_Log_Error , msg);
		    exit(EXIT_FAILURE);
        break;
    }
	
	return -1;
}









/*------------------------------------------------------*/
//  IFFT
/*------------------------------------------------------*/
void calcIFFT_TF(  struct zip  *handle ,
				char        *fname[] ,
				int         f_max )   //file総数
{

	//printf("calcIFFT_TF start\n");

	struct zip_stat zs;	
	struct zip_file* fp;
    char  tag[33];
    char  type[33];
    int   dim;
    int   rows;
    int   cols;
	int   i;	
	int   len;
	char  msg[MSG_LEN];
    int   src;
	int   mic;
	int   fft;



	/*------------------------------------------------------*/
	// zipから全データを配列に格納
	/*------------------------------------------------------*/
	// f_max:総SRC数
	for( src=0 ; src < f_max ; src++ )
	{

		//printf("%s\n",fname[src]);

		//zip open
		if( ( fp = zip_fopen( handle , fname[src] ,0) ) == NULL)
		{
		    sprintf(msg,"fopen error [%s]", fname[src] );
		    harkio_Log_printf(harkio_Log_Error , msg);
			printf("%s\n",msg);
		    exit(EXIT_FAILURE);
		}
	
		zip_stat_init(&zs);

		zip_stat(handle,  fname[src] ,ZIP_STAT_SIZE ,  &zs);


		//zip header
		zip_fread(fp,tag  ,32);
		zip_fread(fp,type ,32);
		zip_fread(fp,&dim ,4);
		zip_fread(fp,&rows,4);
		zip_fread(fp,&cols,4);




		//伝達関数部分のデータ長
		len = rows * cols * sizeof(COMPLEX_FLOAT);

		COMPLEX_TYPE data[len];


		//get all data
		for( i=0;i< len   ;i++)
		{
			zip_fread(fp, &data[i] , COMPLEX_LEN  ); //COMPLEX_LEN=8
		}

		int cnt = 0;

		//rows:総チャンネル数
		for( mic=0 ; mic < g.Nmic ; mic++)
		{	
			//g.Nfft : 512
			for( fft=0 ; fft< (g.Nfft/2)+1 ; fft++) 
			{
//				g.TmpTFvect[src][mic][fft] = conjf(data[cnt]); //harktool4no baai
				g.TmpTFvect[src][mic][fft] = data[cnt]; 
				cnt++;
			}

		}//end

		zip_fclose(fp);
	}//for end




	for ( src = 0; src < g.Nsrc ; src++)
	{
		for ( mic = 0 ;  mic < g.Nmic ;  mic++)
		{
			memset( float_a  , '\0' , g.Nfft  * sizeof(float) );//512
			memset( g.buff_A , '\0' , g.Nfft  * sizeof(COMPLEX_FLOAT) );//512
		
			for ( fft = 0; fft < (g.Nfft/2)+1 ; fft++)
			{
				g.buff_A[fft] =  g.TmpTFvect[src][mic][fft];
			}


			// fft実行 A[]の値をifftした結果がa[]にセットされる
			 harktool_calcFloatIFFT( g.buff_A ,  //original
                                     float_a  ); //結果

			//Nfft/2 +1分コピー
			for ( fft = 0; fft < g.Nfft ; fft++ )
			{
				g.TFvect[src][mic][fft] =  float_a[fft] + 0 * _Complex_I  ;
			}
		}
	}

}










/*------------------------------------------------------*/
//  座標変換
/*------------------------------------------------------*/
void WriteCoordinate(  char *outfile , int z_axis , int fs)
{

	//printf("WriteCoordinate start\n");


	FILE *fpw;	
	float real;
	float imag;
	char  wbuff[256];
	float z_data;
	char  msg[MSG_LEN];
	int   src;
	int   mic;
	int   fft;
	//int   nfft = g.Nfft/2+1 ;


	//heder write 
	if( (fpw = fopen( outfile  , "w" )) == NULL )
	{
	    sprintf(msg,"fopen error [%s]", outfile );
	    harkio_Log_printf(harkio_Log_Error , msg);
	    exit(EXIT_FAILURE);
	}

	memset(  wbuff , 0 , sizeof(wbuff));
	sprintf( wbuff , "%s\n", "id,mic,x,y,z" );
	fwrite(  wbuff, strlen(wbuff), 1, fpw );


	for( src=0 ; src < g.Nsrc ; src++ )
	{
		for( mic=0 ; mic < g.Nmic ; mic++)
		{	
			for( fft=0 ; fft< g.Nfft ; fft++)//512
			{
				float x_data = fft * (1.0f / fs) ; // TIME[s]
				float y_data = src * 0.2 ;// src or mic

				//z軸 
				real = crealf( g.TFvect[src][mic][fft] );//data[cnt]
				imag = cimagf( g.TFvect[src][mic][fft] );

				z_data = calcZValue( z_axis , real  , imag );

				memset(wbuff,0,sizeof(wbuff));
				sprintf( wbuff ,   "%d , %d , %f , %f , %f\n", src , mic , x_data , y_data , z_data );
				fwrite( wbuff, strlen(wbuff), 1, fpw );
			}

		}//end
	}//for end


	fclose( fpw );


}









void getZipData( struct zip  *handle , char *filename , int *nmic ,  int *nfft ,int *nfs)
{

	//printf("getZipData start\n");

	struct zip_stat  zs;	
	struct zip_file* fp;
    char  tag[33];
    char  type[33];
    int   dim;
	char  msg[MSG_LEN];
	int rows;
	int cols;
	


	//zip open
	if( ( fp = zip_fopen( handle ,filename ,0) ) == NULL)
	{
	    sprintf(msg,"fopen error [%s]", filename);
	    harkio_Log_printf(harkio_Log_Error , msg);
	    exit(EXIT_FAILURE);
	}

	zip_stat_init(&zs);

	zip_stat(handle,  filename ,ZIP_STAT_SIZE ,  &zs);


	//zip header
	zip_fread(fp,tag  ,32);
	zip_fread(fp,type ,32);
	zip_fread(fp,&dim ,4);
	zip_fread(fp,&rows,4);
	zip_fread(fp,&cols,4);
	


	//get Config
	char *str = "transferFunction/source.xml";
	harkio_XML* src_xml = readZippedXML( handle , str );
	if(src_xml==NULL) 
	{
	    sprintf(msg,"fopen error [%s]", str);
	    harkio_Log_printf(harkio_Log_Error , msg);
	    exit(EXIT_FAILURE);
	}


	//
	char *str1 = "transferFunction/microphones.xml";
	harkio_XML* mic_xml = readZippedXML( handle , str1 );
	if(mic_xml==NULL) 
	{
	    sprintf(msg,"fopen error [%s]", str1);
	    harkio_Log_printf(harkio_Log_Error , msg);
	    exit(EXIT_FAILURE);
	}

	*nfft = src_xml->cfg->nfft;
	*nmic = mic_xml->poses->size;
	*nfs  = src_xml->cfg->samplingRate;

	free(src_xml);
	free(mic_xml);
	zip_fclose(fp);

	
}	













/*-----------------------------------------------------------------------------*
* 初期化
*
*
*
*-----------------------------------------------------------------------------*/
void global_init()
{


    g.argv.tf_type             = 0                                        ; // 引数 伝達関数種別 MUSIC /GSS
    g.argv.direct_sound_length = 0                                        ;
    g.argv.normalize_src       = 0                                        ; // 正規化(SRC) 引数
    g.argv.normalize_mic       = 0                                        ; // 正規化(mic) 引数
    g.argv.normalize_freq      = 0                                        ; // 正規化(freq)引数
    g.argv.mch                 = 1                                        ; // mouthTSP

    //	g.argv.cutLow_freq         = 0                                        ; //引数 CalcSysImpで使用
//	g.argv.cutHigh_freq        = 0                                        ; //引数　CalcSysImpで使用

    memset( g.argv.mic_file     , '\0' , sizeof(g.argv.mic_file))     ; // mic file (引数)
    memset( g.argv.tsp_file     , '\0' , sizeof(g.argv.tsp_file))     ; // TSP List file (引数)
    memset( g.argv.impulse_file , '\0' , sizeof(g.argv.impulse_file)) ; // ImpulseList file      (引数)
    memset( g.argv.noise_file   , '\0' , sizeof(g.argv.noise_file))   ; // noise file (引数)
    memset( g.argv.input        , '\0' , sizeof(g.argv.input))        ; // 引数 ( tsp001.wav)
    memset( g.argv.output       , '\0' , sizeof(g.argv.output))       ; // 引数 作成する伝達関数のzip (hark.zip)



    g.Nsrc             = 0                                                ; // 音源数
    g.Nmic             = 0                                                ; // mic数
    g.Nfft             = 0                                                ; // fft数 // harkio_Config
    g.SyncAverage      = 0                                                ; // harkio_Config_get
    g.TspOffset        = 0                                                ; // harkio_Config_get
    g.PeakSearchFrom   = 0                                                ; // harkio_Config_get
    g.PeakSearchTo     = 0                                                ; // harkio_Config_get
    g.SamplingRate     = 0                                                ; // harkio_Config_get
    g.SignalMax        = 0                                                ; // harkio_Config_get
    g.TspLength        = 0                                                ; // harkio_Config_get
    g.OrgTspPath       = NULL                                             ;
    memset( g.ChannelList , '\0' , sizeof(g.ChannelList) )                ;
    g.TFvect           = NULL                                             ; // 伝達関数領域
    g.TmpTFvect        = NULL                                             ; // 伝達関数 作業領域
    g.TFsize           = 0                                                ; // 伝達関数領域のsize(Nsrc * Nmic * ( Nfft/2 + 1) )
    g.buff_a           = NULL                                             ; // fft前のデータ
    g.buff_A           = NULL                                             ; // fft後のデータ
    g.buff_w           = NULL                                             ; //
    g.IRvect           = NULL                                             ; // インパルス応答格納領域
    g.TmpIRvect        = NULL                                             ; // インパルス応答 計算用TEMP領域
    g.IRsize           = 0                                                ; // インパルス応答格納領域のsize(Nsrc * nmix * Nfft)
    g.mic_xml          = NULL                                             ; // 伝達関数作成時のマイク位置
    g.tsp_xml          = NULL                                             ; // 伝達関数作成時の
    g.impulse_xml      = NULL                                             ; // 伝達関数作成時の
    g.noise_xml        = NULL                                             ; // 伝達関数作成時の;
    g.src_xml          = NULL                                             ; // 定位した音源（音源定位結果保存形式）
    g.MicPosVect       = NULL                                             ; // mic position 格納領域
    g.SrcPosVect       = NULL                                             ; // src position 格納領域
    memset( g.PosMean , '\0' , sizeof(g.PosMean ) )                         ; // 平均値
    g.Loctf            = NULL                                             ; // 定位伝達関数格納
    g.Septf            = NULL                                             ; // 分離伝達関数格納
    g.TF               = NULL                                             ; // 伝達関数
    g.mic_xml          = NULL                                             ; // 伝達関数作成時のマイク位置
    g.tsp_xml          = NULL                                             ; // 伝達関数作成時の
    g.impulse_xml      = NULL                                             ; // 伝達関数作成時の
    g.noise_xml        = NULL                                             ; // 伝達関数作成時の;
    g.src_xml          = NULL                                             ; // 定位した音源（音源定位結果保存形式）
    memset( g.tmp.date_time , '\0' , BUFFSIZE )                           ; // ディレクトリ名作成用現在日時分秒
    memset( g.tmp.tsp_path  , '\0' , BUFFSIZE )                           ; // tspファイル格納パス   TSP storage path
    memset( g.tmp.ir_path   , '\0' , BUFFSIZE )                           ; // irファイル格納パス
    memset( g.tmp.tf_path   , '\0' , BUFFSIZE )                           ; // tfファイル格納パス
    g.tmp.flt_path         = NULL                                             ; // CalcSysimpで作成したインパルス応答ファイル格納path名(Nsrc分)
    g.tmp.tsp_wav_name     = NULL                                             ; // TSPファイル名(Nsrc分)
    memset( g.tmp.outimpfilehead ,'\0', PATH_LEN )                        ; // CalcSysyImpで使用するpath+header部
    g.wav.file_size   = 0                                                ;
    g.wav.total_frame = 0                                                ;
    g.wav.data        = NULL                                             ;
    g.tmp.flt_path     = NULL                                             ; // CalcSysimpで作成したインパルス応答ファイル格納path名(Nsrc分)
    g.tmp.tsp_wav_name = NULL                                             ; // TSPファイル名(Nsrc分)

}





